/*
ABC4F, published in:
Foll et al. 2008 Genetics, 179 (2). 
Copyright (C) Matthieu Foll

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#include <stdio.h>
#include <stdlib.h>
#include <iomanip>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <time.h>
#include <math.h>
#include <algorithm>
#include <cstdlib>

#include "anyoption.h"

#include <gsl/gsl_multifit.h>
// you need to have gsl installed and include the correct directory into the path
// http://www.gnu.org/software/gsl/ or http://gnuwin32.sourceforge.net/packages/gsl.htm for a windows version

#define C(i) (gsl_vector_get(c,(i)))
#define Y(i) (gsl_vector_get(y,(i)))
#define Xs(i,j) (gsl_matrix_get(X,(i),(j)))


#include "MersenneTwister.h"
#include "pthread.h"
// you need to have the pthread library installed and include the correct directory into the path
// generally installed on unix machine http://www.gnu.org/software/pth/,
// see http://sources.redhat.com/pthreads-win32/ for a windows implementation

// compilation should be somthing like that:
// g++ -L where_gsl_is_installed/gsl/lib/ -lpthread -lgsl -lgslcblas -lm   -I where_gsl_is_installed/gsl/include/ -o ABC4F main.cpp anyoption.cpp

#define max_nr_pop 60
#define max_nr_loci  5000
#define max_nr_stats 1000

#define Pi 3.1415926535897932385

using namespace std;

void *start(void *p_data);
void read_input(ifstream& infile);
void calculate_stats(float *res_vect,double hpdi_pc,int nb_final,double *m,double *var,double *hpdi_low,double *hpdi_high,double *mode);


struct  locus_data
    {
    int   n; // total number of alleles in the sample = n_ij
    int   nA1;  // observed allele frequency spectrum = nA1_ij
    double   p; // estimated allele frequency = p_ij
    } ;

struct  pop_data
    {
    locus_data  locus[max_nr_loci]; // alleles count
    }  ;

int I;
int J;
//int indiv=100;

int nb_class_fis=15;
int nb_class_fst=25;

pop_data *pop_target;
pop_data *pop;

int nbreps;
bool pilot=true;

int nb_final=5000;
double ratio=0.005;

int rep=0;

int last_printf=0;

ofstream *outfile_fis;
ofstream *outfile_fst;

double **v_dist_fis;
double **v_dist_fst;

double *quantile_fis; // fis quantile
double *quantile_fst; // fst quantiles

int nb_procs=1; // number of threads

double target_fis[max_nr_stats]; // target stats for fis
double target_fst[max_nr_stats]; // target stats for fst

int *nb_fis; // nb of simulations carried out for fis
int *nb_fst;//=new int[J]; // and for fst

bool finished=false; // indicated if all parameters have exactly nb_final simulations calculted

float hpdi_pc=0.95;

int bias_hidden=1;
int bias_fixed=0;

string file_prefix;
string odir="";
string file_name;
string stats_prefix;

pthread_mutex_t mutex_gauge=PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t mutex_outfile=PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t mutex_randseed=PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t mutex_counter=PTHREAD_MUTEX_INITIALIZER;

// return the index of the smalest element in array tab[0..size-1]
int minindex(double *tab,int size)
{
int i=0;
double min=tab[0];
for (int j=1;j<size;j++)
{
 if (tab[j]<min)
        {
        i=j;
        min=tab[j];
        }
}
return i;
}

inline int round2(double x)
{
return int(x > 0.0 ? x + 0.5 : x - 0.5);
}

/* 1. CREATE AN OBJECT */
AnyOption *opt = new AnyOption();

int main(int argc, char **argv)
{


    /* 2. SET PREFERENCES  */
    opt->noPOSIX(); /* do not check for POSIX style character options */

    /* 3. SET THE USAGE/HELP   */
    opt->addUsage( "" );
    opt->addUsage( "Usage: " );
    opt->addUsage( "" );
    opt->addUsage( " -help        Prints this help " );
    opt->addUsage( " -nfis 15     Number of stats for Fis, default is 15 " );
    opt->addUsage( " -nfst 25     Number of stats for Fst, default is 25 " );
    opt->addUsage( " -n 5000      Number of output data sets, default is 5000 " );
    opt->addUsage( " -ar 0.005    Acceptance ratio in ABC algorithm, default is 0.005 " );
    opt->addUsage( " -nprocs 2    Number of processors used, default is 1 " );
    opt->addUsage( " -o alleles   Output file prefix, default is input file without the extension " );
    opt->addUsage( " -od .        Output directory, default is the same as program file  " );
    opt->addUsage( " -s stats.txt Statistics file, no statistics output by default" );
    opt->addUsage( " -hpdi 0.95   HDPI percentage if -s is defined, 0.95 by default " );
    opt->addUsage( " -keep        Keep output files with simulated summary statistics " );
    opt->addUsage( " -convert     Convert data as codominant markers based on Fis estimates " );
    opt->addUsage( " -bh          Minimum total nb of indivs with a band, default is 1 for AFLP " );
    opt->addUsage( " -bf          Minimum total nb of indivs without a band, default is 0 for AFLP " );
    opt->addUsage( " alleles.txt  Name of the input file " );
    opt->addUsage( "" );

    /* 4. SET THE OPTION STRINGS/CHARACTERS */
    opt->setFlag(  "help");
    opt->setFlag(  "keep");
    opt->setFlag(  "convert");
    opt->setOption(  "nfis" );
    opt->setOption(  "nfst" );
    opt->setOption(  "n" );
    opt->setOption(  "ar" );
    opt->setOption(  "o" );
    opt->setOption(  "od" );
    opt->setOption(  "s" );
    opt->setOption(  "hpdi" );
    opt->setOption(  "bh" );
    opt->setOption(  "bf" );
    opt->setOption(  "nprocs" );

    /* 5. PROCESS THE COMMANDLINE AND RESOURCE FILE */
	/* go through the command line and get the options  */
    opt->processCommandArgs( argc, argv );

    if( ! opt->hasOptions() || opt->getArgc()==0) { /* print usage if no options */
		opt->printUsage();
	        delete opt;
		exit(0);
	}

	 /* 6. GET THE VALUES */
    if( opt->getValue( "s" ) == NULL && opt->getFlag( "convert" ))
	{
		cout << "Statistics calculation required to convert to codominant data (use -s option)" << endl;
		exit(0);
	}


    if( opt->getFlag( "help" ) || opt->getFlag( 'h' ) )
        opt->printUsage();

	if( opt->getValue( "nfis" ) != NULL )
        nb_class_fis=atoi(opt->getValue( "nfis" ));

    if( opt->getValue( "nfst" ) != NULL )
        nb_class_fst=atoi(opt->getValue( "nfst" ));

    if( opt->getValue( "n" ) != NULL )
        nb_final=atoi(opt->getValue( "n" ));

    if( opt->getValue( "nprocs" ) != NULL )
        nb_procs=atoi(opt->getValue( "nprocs" ));

    if( opt->getValue( "ar" ) != NULL )
        ratio=atof(opt->getValue( "ar" ));

    if( opt->getValue( "s" ) != NULL )
        stats_prefix=opt->getValue( "s" );

    if( opt->getValue( "hpdi" ) != NULL )
        hpdi_pc=atof(opt->getValue( "hpdi" ));

    if( opt->getValue( "bh" ) != NULL )
        bias_hidden=atoi(opt->getValue( "bh" ));

    if( opt->getValue( "bf" ) != NULL )
        bias_fixed=atoi(opt->getValue( "bf" ));

    /*FILE *pp;
    char buf[256];

    if (!(pp = popen("qstat -a | grep `whoami` | awk '{print $7}'", "r") )) // unix : "grep \"primary processor\" /proc/cpuinfo | wc -l"
    {								    // healthphy : "qstat -a | grep `whoami` | awk '{print $7}'"
        cout << "Error while launching external command";
        exit(0);
    }

    else
    {
        while (fgets(buf, sizeof buf, pp))
        {   }
        pclose(pp);

        istringstream read_cmd(buf);

        if(read_cmd >> nb_procs;)
           cout << nb_procs << " processor(s) detected" << endl;
        else {cout << "Number of processors: " ; cin >> nb_procs;}
    }*/

    // real data set
    pop_target=new pop_data[max_nr_pop];
    ifstream alleles_file;

// begin files batch work
for (int f_index=0;f_index<opt->getArgc();f_index++)
{
    file_name=opt->getArgv(f_index);

   // cout << "Treating file " << file_name << endl;

    if( opt->getValue( "od" ) != NULL )
        odir=opt->getValue( "od" );

    if( opt->getValue( "o" ) != NULL )
        file_prefix=opt->getValue( "o" );
    else
    {
        file_prefix=file_name;
        if(file_prefix.find(".",1)!= string::npos)
            file_prefix=file_prefix.substr(0,file_prefix.length()-4);
        int position;
        if(position=file_prefix.find_last_of('/'))
            file_prefix=file_prefix.substr(position+1,file_prefix.length());
        //cout << file_prefix << endl;

    }

    // real data set input file
    alleles_file.open(file_name.c_str());
    if(!alleles_file.is_open())
    {
	cout << "Could not open file " << file_name << endl;
	exit(0);
    }
    read_input(alleles_file);

    pop=new pop_data[J];
    outfile_fst=new ofstream[J];
    outfile_fis=new ofstream[J];
    nb_fst=new int[J];
    nb_fis=new int[J];

	// create vectors of distance for pilot runs
	int nb_pilot=(int)(nb_final/(10*ratio));

	v_dist_fis=new double*[J];
	for (int j=0;j<J;j++)
		v_dist_fis[j]=new double[nb_pilot];

	v_dist_fst=new double*[J];
	for (int j=0;j<J;j++)
		v_dist_fst[j]=new double[nb_pilot];

    // create the output files
    for (int j=0;j<J;j++)
	{
	ostringstream oss;
	oss << odir << file_prefix << "_Fis" << (j+1) << ".txt";
	//cout << oss.str() << endl;
	outfile_fis[j].open(oss.str().c_str());
	}

    for (int j=0;j<J;j++)
	{
	ostringstream oss;
	oss << odir << file_prefix << "_Fst" << (j+1) << ".txt";
	//cout << oss.str() << endl;
	outfile_fst[j].open(oss.str().c_str());
	}

     pthread_t *threads=new pthread_t[nb_procs];

	 time_t time1,time2,time_pilot;

     time1=time(NULL);

	 nbreps=nb_pilot;
     rep=0;
     pilot=true;
     finished=false;

     cout << "Pilot run to determine threshold acceptance value..." << endl;

    /* Create independent threads each of which will execute function */
     for(int j=0;j<nb_procs;j++)
        {
        if(pthread_create( &threads[j], NULL,start,NULL))
        cout << "Warning : could not create thread number " << j+1 << endl;
        }

     /* Wait till threads are complete before main continues */
     for(int j=0;j<nb_procs;j++)
        pthread_join( threads[j], NULL);

    time_pilot=time(NULL);

	pilot=false;

	nbreps=(int)(nb_final/ratio);
	rep=0;

    last_printf=0;
    cout << endl << endl << "Calculation, it should take about " << 10*(time_pilot-time1)/60 << " minutes..." << endl;

	time2=time(NULL);

	quantile_fis=new double[J];
	for (int j=0;j<J;j++)
	{
		nb_fis[j]=0;
		sort(&v_dist_fis[j][0],&v_dist_fis[j][nb_pilot]);
		quantile_fis[j]=v_dist_fis[j][(int)(ratio*nb_pilot)];
	}

	quantile_fst=new double[J];
	for (int j=0;j<J;j++)
	{
		nb_fst[j]=0;
		sort(&v_dist_fst[j][0],&v_dist_fst[j][nb_pilot]);
		quantile_fst[j]=v_dist_fst[j][(int)(ratio*nb_pilot)];
	}

	// full run to output data
	time1=time(NULL);

    /* Create independent threads each of which will execute function */
     for(int j=0;j<nb_procs;j++)
        {
        if(pthread_create( &threads[j], NULL,start,NULL))
        cout << "Warning : could not create thread number " << j+1 << endl;
        }

     /* Wait till threads are complete before main continues */
     for(int j=0;j<nb_procs;j++)
        pthread_join( threads[j], NULL);

    // close files
    for (int j=0;j<J;j++)
	{
    	outfile_fis[j].close();
        outfile_fis[j].clear();
	}

    for (int j=0;j<J;j++)
	{
    	outfile_fst[j].close();
        outfile_fst[j].clear();
	}

// regression
    cout << endl << endl << "Performing regression... " ;
    string line;
    FILE *fp;

    ofstream outfile;
    ofstream outfile_no;

    outfile.open((odir+file_prefix+"_regression.txt").c_str());
    outfile_no.open((odir+file_prefix+"_no_regression.txt").c_str());

    gsl_matrix *X, *cov; // solve c in y=X*c with weigth w
    gsl_vector *y, *w, *c; // cov output covariance matrix
    double chisq;

    float dist; // current values in the file
    double fis;
    double fst;
    float stat;

    float **res_fis=new float*[J];
    float **res_fis_no=new float*[J];

    cout << endl << "Fis: population ";
    // read the file and make the regression for fis
   for(int j=0;j<J;j++)
   {
       	ostringstream oss;
        oss << odir << file_prefix << "_Fis" << (j+1) << ".txt";
        if ((fp=fopen(oss.str().c_str(), "r")) == NULL)
        {
            printf("Error opening file\n");
            exit(0);
        }

    cout << j+1 << ", ";

    X = gsl_matrix_alloc (nb_final, nb_class_fis+1);
    y = gsl_vector_alloc (nb_final);
    w = gsl_vector_alloc (nb_final);

    c = gsl_vector_alloc (nb_class_fis+1);
    cov = gsl_matrix_alloc (nb_class_fis+1, nb_class_fis+1);

        res_fis[j]=new float[nb_final];
        res_fis_no[j]=new float[nb_final];

    // create  X, y and w from file
    for (int l=0;l<nb_final;l++)
    {
        gsl_matrix_set (X, l, 0, 1.0);

        fscanf(fp, "%f", &dist);
        gsl_vector_set (w, l,1-( (dist*dist)/quantile_fis[j]) );

        fscanf(fp, "%lf", &fis);
        if (fis==1) fis=0.99999999; // can happend due to rouding in output/input file
        if (fis==0) fis=0.00000001;
        gsl_vector_set (y, l,log(fis/(1-fis)));
        res_fis_no[j][l]=fis;

            for (int k=0;k<nb_class_fis;k++)
                {
                fscanf(fp, "%f", &stat);
                gsl_matrix_set (X, l, k+1, stat);
                }
    }

    fclose(fp);
    feof(fp);

    // make the regression
    gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (nb_final, nb_class_fis+1);
    gsl_multifit_wlinear (X, w, y, c, cov,&chisq, work);
    gsl_multifit_linear_free (work);

    // calculate modified values
    double reg;

    for (int l=0;l<nb_final;l++)
    {
        reg=Y(l);

        for (int k=0;k<nb_class_fis;k++)
            reg-=Xs(l,k+1)*C(k+1);

        res_fis[j][l]=exp(reg)/(1+exp(reg));
    }

  }


   // read the file and make the regression for fst
    float **res_fst=new float*[J];
    float **res_fst_no=new float*[J];

    cout << endl << "Fst: population ";

   for(int j=0;j<J;j++)
   {
       	ostringstream oss;
        oss << odir  << file_prefix << "_Fst" << (j+1) << ".txt";
        if ((fp=fopen(oss.str().c_str(), "r")) == NULL)
        {
            printf("Error opening file\n");
            exit(0);
        }

        cout << j+1 << ", ";

        X = gsl_matrix_alloc (nb_final, nb_class_fst+1);
        y = gsl_vector_alloc (nb_final);
        w = gsl_vector_alloc (nb_final);

        c = gsl_vector_alloc (nb_class_fst+1);
        cov = gsl_matrix_alloc (nb_class_fst+1, nb_class_fst+1);

        res_fst[j]=new float[nb_final];
        res_fst_no[j]=new float[nb_final];

        // create  X, y and w from file
        for (int l=0;l<nb_final;l++)
        {
            gsl_matrix_set (X, l, 0, 1.0);

            fscanf(fp, "%f", &dist);
            gsl_vector_set (w, l,1-( (dist*dist)/quantile_fst[j] ) );

            fscanf(fp, "%lf", &fst);
            if (fst==1) fst=0.99999999; // can happend due to rouding in output/input file
            if (fst==0) fst=0.00000001;
            gsl_vector_set (y, l,log(fst/(1-fst)));
            res_fst_no[j][l]=fst;

            for (int k=0;k<nb_class_fst;k++)
                {
                fscanf(fp, "%f", &stat);
                gsl_matrix_set (X, l, k+1, stat);
                }
        }

        fclose(fp);
        feof(fp);

        // make the regression
        gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (nb_final, nb_class_fst+1);
        gsl_multifit_wlinear (X, w, y, c, cov,&chisq, work);
        gsl_multifit_linear_free (work);

        // calculate modified values
        double reg;

        for (int l=0;l<nb_final;l++)
        {
            reg=Y(l);

            for (int k=0;k<nb_class_fst;k++)
                reg-=Xs(l,k+1)*C(k+1);

            res_fst[j][l]=exp(reg)/(1+exp(reg));
        }
   }
// remove files if keep is not asked
if(!opt->getFlag( "keep" ))
{
    for (int j=0;j<J;j++)
	{
        ostringstream oss;
        oss << odir << file_prefix << "_Fis" << (j+1) << ".txt";
        remove(oss.str().c_str());
	}

    for (int j=0;j<J;j++)
	{
        ostringstream oss;
        oss << odir << file_prefix << "_Fst" << (j+1) << ".txt";
        remove(oss.str().c_str());
	}
}


// write out full results
    for (int l=0;l<nb_final;l++)
    {

        for (int j=0;j<J;j++)
            {
                outfile << " " << res_fis[j][l];
                outfile_no << " " << res_fis_no[j][l];
            }

        for (int j=0;j<J;j++)
            {
                outfile << " " << res_fst[j][l];
                outfile_no << " " << res_fst_no[j][l];
            }
        outfile << endl;
        outfile_no << endl;
    }

	outfile.close();
	outfile.clear();
	outfile_no.close();
	outfile_no.clear();

// end of regression

double *final_fis=new double[J];

// write out statistics : mean, mode, variance and 95% HPDI.
    if( opt->getValue( "s" ) != NULL )
    {
        double m,var,hpdi_low,hpdi_high,mode;
        ofstream out_stats;
        ofstream out_stats_mode;

        //out_stats_mode.open((odir+stats_prefix+"_modes.txt").c_str(), ios::app);
        out_stats.open((odir+stats_prefix).c_str());
        out_stats << "      mean " << " mode" << " variance " << round(hpdi_pc*100) <<  "% HPDI" << endl;

  	for (int j=0;j<J;j++)
        {
            //ostringstream oss;
            //oss << odir << stats_prefix << "_Fis" << (j+1) << ".txt";
            //out_stats.open(oss.str().c_str(),ios::app);
            calculate_stats(res_fis[j],hpdi_pc,nb_final,&m,&var,&hpdi_low,&hpdi_high,&mode);
            out_stats << "Fis" << (j+1) <<  setprecision(3) << " " <<  m << " " << mode <<  " " << var << " [" << hpdi_low << ";" << hpdi_high << "]" << endl;
            //out_stats_mode << " " << mode ;
            //out_stats.close();
            //out_stats.clear();
        }

        for (int j=0;j<J;j++)
        {
            //ostringstream oss;
            //oss << odir << stats_prefix << "_Fst" << (j+1) << ".txt";
            //out_stats.open(oss.str().c_str(),ios::app);
            calculate_stats(res_fst[j],hpdi_pc,nb_final,&m,&var,&hpdi_low,&hpdi_high,&mode);
            out_stats << "Fst" << (j+1) <<  setprecision(3) << " "  << m << " " << mode <<  " " << var << " [" << hpdi_low << ";" << hpdi_high << "]" <<endl;
            //out_stats_mode << " " << mode ;
            //out_stats.close();
           // out_stats.clear();
        }
        //out_stats_mode << endl;
    }
// end of statistics calculation

// write out data converted as codominant markers
if(opt->getFlag( "convert" ))
{
	ofstream out_codom;
	out_codom.open((odir+file_prefix+"_codominant.txt").c_str());
	out_codom << "#value of Fis used to convert from dominant data: " << endl;
	for(int j=0;j<J;j++)
	{
		out_codom << "#Population " << j+1 << ": " <<  final_fis[j] << endl;
	}
	out_codom << endl << "[loci]=" << I  << endl << endl;

	out_codom << "[populations]=" << J << endl << endl;

	double g2,q;
	int nA1,nA2;

	for(int j=0;j<J;j++)
	{
		outfile << "[pop]=" << j+1 << endl;
        	for(int i=0;i<I;i++)
        	{
			g2=((double)(pop_target[j].locus[i].n-pop_target[j].locus[i].nA1))/((double)pop_target[j].locus[i].n);
			q=(-final_fis[j]+sqrt(final_fis[j]*final_fis[j]+4*g2*(1-final_fis[j])))/(2*(1-final_fis[j]));
			nA2=(int)round2(q*pop_target[j].locus[i].n);
			nA1=pop_target[j].locus[i].n-nA2;
         		out_codom << setw(3) << i+1 << "  " << pop_target[j].locus[i].n << "  2  " << setw(3) << nA1 << " " << nA2  << endl;
        	}
        out_codom << endl;
	}
	out_codom.close();
}


    time2=time(NULL);

    cout << "done!" << endl << endl << "Job made in " << time2-time1 << " seconds" << endl;
} // end of files batch work
     exit(0);
}


void *start(void *p_data)
{

pop_data *pop=new pop_data[J];

double  fis[max_nr_pop]; // within population inbreeding coefficient

int  I_sim;

double  fst[max_nr_pop];

double  a;

bool  locus_fixed[max_nr_loci]; // indicate if a locus is fixed 1 in all populations
bool  locus_hidden[max_nr_loci]; // indicate if a locus is fixed 0 in all populations

double  freq_ancestral[max_nr_loci];

double  theta[max_nr_pop];

int nb_fixed;
int nb_hidden;

double  g; // genotype frequency

string line;

MTRand randgen;

int simus=0;

double freq[max_nr_loci]; // for quantile estimation

double cur_stat_fis[max_nr_stats]; // current stat for fis

double cur_stat_fst[max_nr_stats]; // current stat for fst

int max_wait=0;
int nb_waited=0;

int local_rep;

ostringstream *write_line_fis=new ostringstream[J];
for (int j=0;j<J;j++)
	write_line_fis[j].str("" );

ostringstream *write_line_fst=new ostringstream[J];
for (int j=0;j<J;j++)
	write_line_fst[j].str("" );

// mutex to seed rand generator
pthread_mutex_lock( &mutex_randseed );
randgen.seed();
pthread_mutex_unlock( &mutex_randseed );


// simulate
while (rep < nbreps || (!pilot && !finished))
{

pthread_mutex_lock( &mutex_counter );
rep++;
/*if (100*rep/nb_final>last_printf)
{
    last_printf=100*rep/nb_final
    cout << last_printf << "% ";
}*/
local_rep=rep-1;
pthread_mutex_unlock( &mutex_counter );

simus++;

for (int j=0;j<J;j++)
	fis[j]=randgen.randDblExc();

for (int j=0;j<J;j++)
{
fst[j]=randgen.randDblExc();
theta[j]=(1.0/fst[j])-1.0;
}

a=0.1+exp(randgen.randNorm(0.0,1.0));//0.1+randgen.randDblExc(1.9);

// initialize ancestral allele frequencies from beta distribution
// initialize allele frequencies from beta(theta*pi,theta*(1-pi))
// and phenotype count from binomial(g,n)

I_sim=0;
nb_fixed=0;
nb_hidden=0;
int sum_pops[max_nr_loci];
int sum_pops_indiv[max_nr_loci];

do {

    freq_ancestral[I_sim]=randgen.randBeta(a,a);    // initialize ancestral allele frequencies from beta distribution

    if (freq_ancestral[I_sim]==1) freq_ancestral[I_sim]=0.9999999999999999;

    locus_fixed[I_sim]=true;
    locus_hidden[I_sim]=true;
    sum_pops[I_sim]=0;
    sum_pops_indiv[I_sim]=0;

    for(int j=0;j<J;j++)
    {

    pop[j].locus[I_sim].p=randgen.randBeta(theta[j]*freq_ancestral[I_sim],theta[j]*(1-freq_ancestral[I_sim]));

    g= pop[j].locus[I_sim].p*pop[j].locus[I_sim].p
                   +2*pop[j].locus[I_sim].p*(1-pop[j].locus[I_sim].p)*(1-fis[j])
                   +fis[j]*pop[j].locus[I_sim].p*(1-pop[j].locus[I_sim].p);

    pop[j].locus[I_sim].n=pop_target[j].locus[I_sim-nb_fixed-nb_hidden].n;

    pop[j].locus[I_sim].nA1=randgen.Binomial((int32) pop[j].locus[I_sim].n,g);

    //locus_fixed[I_sim]=locus_fixed[I_sim]&&(pop[j].locus[I_sim].nA1==pop[j].locus[I_sim].n);
    //locus_hidden[I_sim]=locus_hidden[I_sim]&&(pop[j].locus[I_sim].nA1==0);
    sum_pops[I_sim]+=pop[j].locus[I_sim].nA1;
    sum_pops_indiv[I_sim]+=pop[j].locus[I_sim].n;
    }

    locus_fixed[I_sim]=(sum_pops[I_sim]>sum_pops_indiv[I_sim]-bias_fixed);
    locus_hidden[I_sim]=(sum_pops[I_sim]<bias_hidden);

    nb_fixed+=locus_fixed[I_sim];
    nb_hidden+=locus_hidden[I_sim];

    I_sim++;
    }

while (I_sim-nb_fixed-nb_hidden<I);

// calculate summary statistics for Fis
double dist_fis[max_nr_pop]; // distance between stats and target stats
//if(!opt->getFlag( "comfis" ))
//{
	for (int j=0;j<J;j++)
	{
    		dist_fis[j]=0;
    		int i2=0;
    		for (int i=0;i<I_sim;i++)
        	{
                	 if(!locus_fixed[i] && !locus_hidden[i])
                	{
                    		freq[i2]=((double)pop[j].locus[i].nA1)/((double)pop[j].locus[i].n);
                    		i2++;
                	}
        	}

    		sort(&freq[0],&freq[I]);

		for (int k=0;k<nb_class_fis;k++)
    		{
		        cur_stat_fis[k+j*nb_class_fis]=freq[(int)(I*(k+1)/(nb_class_fis+1))];
        		dist_fis[j]+=(cur_stat_fis[k+j*nb_class_fis]-target_fis[k+j*nb_class_fis])*(cur_stat_fis[k+j*nb_class_fis]-target_fis[k+j*nb_class_fis]);
    		}
	}
/*}
else
{
	for (int j=0;j<J;j++)
	{
    		dist_fis[j]=0;
    		int i2=0;
    		for (int i=0;i<I_sim;i++)
        	{
                	 if(!locus_fixed[i] && !locus_hidden[i])
                	{
                    		freq[i2]=(((double)sum_pops[i])/((double)sum_pops_indiv[i]));
                    		i2++;
                	}
        	}

    		sort(&freq[0],&freq[I]);

		for (int k=0;k<nb_class_fis;k++)
    		{
		        cur_stat_fis[k+j*nb_class_fis]=freq[(int)(I*(k+1)/(nb_class_fis+1))];
        		dist_fis[j]+=(cur_stat_fis[k+j*nb_class_fis]-target_fis[k+j*nb_class_fis])*(cur_stat_fis[k+j*nb_class_fis]-target_fis[k+j*nb_class_fis]);
    		}
	}
}*/


// calculate difference between pop_j and sum_pops, sum_pops calculated during simulation process just above
double dist_fst[max_nr_pop];
for (int j=0;j<J;j++)
{
    dist_fst[j]=0;
    int i2=0;
    for (int i=0;i<I_sim;i++)
        {
                 if(!locus_fixed[i] && !locus_hidden[i])
                {
                    if(sum_pops[i]!=0)
                    freq[i2]=(((double)pop[j].locus[i].nA1)/((double)pop[j].locus[i].n) - ((double)sum_pops[i])/((double)sum_pops_indiv[i])) / (((double)sum_pops[i])/((double)sum_pops_indiv[i])) ;
                    else freq[i2]=0;
                    i2++;
                }
        }

    sort(&freq[0],&freq[I]);

    for (int k=0;k<nb_class_fst;k++)
    {
        cur_stat_fst[k+j*nb_class_fst]=freq[(int)(I*(k+1)/(nb_class_fst+1))];
	//cur_stat_fst[k+j*nb_class_fst]=cur_stat_fis[k+j*nb_class_fis];
        dist_fst[j]+=(cur_stat_fst[k+j*nb_class_fst]-target_fst[k+j*nb_class_fst])*(cur_stat_fst[k+j*nb_class_fst]-target_fst[k+j*nb_class_fst]);
    }
}

if (!pilot && !finished)
{
	bool temp_finished=true;

	for (int j=0;j<J;j++)
	{
		if (dist_fis[j]<=quantile_fis[j])
			{
			pthread_mutex_lock( &mutex_counter );
			nb_fis[j]++;
			pthread_mutex_unlock( &mutex_counter );
			}
		temp_finished=temp_finished && (nb_fis[j]>=nb_final);
	}
	for (int j=0;j<J;j++)
	{
		if (dist_fst[j]<=quantile_fst[j])
			{
			pthread_mutex_lock( &mutex_counter );
			nb_fst[j]++;
			pthread_mutex_unlock( &mutex_counter );
			}
		temp_finished=temp_finished && (nb_fst[j]>=nb_final);
	}
	pthread_mutex_lock( &mutex_counter );
	finished=temp_finished;
	pthread_mutex_unlock( &mutex_counter );


	// write out stats and parameters
	for (int j=0;j<J;j++)
	{
		if (dist_fis[j]<=quantile_fis[j]  && nb_fis[j]<=nb_final)
			write_line_fis[j]  << " " << sqrt(dist_fis[j]);
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fst[j]<=quantile_fst[j]  && nb_fst[j]<=nb_final)
			write_line_fst[j]  << " " << sqrt(dist_fst[j]);
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fis[j]<=quantile_fis[j]  && nb_fis[j]<=nb_final)
			write_line_fis[j]  << " " << fis[j];
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fst[j]<=quantile_fst[j]  && nb_fst[j]<=nb_final)
			write_line_fst[j]  << " " << fst[j];
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fis[j]<=quantile_fis[j]  && nb_fis[j]<=nb_final)
		{
		    for (int k=0;k<nb_class_fis;k++)
    			    write_line_fis[j]  << " " << setprecision(5) << cur_stat_fis[k+j*nb_class_fis]-target_fis[k+j*nb_class_fis];
		}
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fst[j]<=quantile_fst[j]  && nb_fst[j]<=nb_final)
		{
		    for (int k=0;k<nb_class_fst;k++)
    			    write_line_fst[j]  << " " << setprecision(5) << cur_stat_fst[k+j*nb_class_fst]-target_fst[k+j*nb_class_fst];
           	}
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fis[j]<=quantile_fis[j]  && nb_fis[j]<=nb_final)
			write_line_fis[j] << endl;
	}

	for (int j=0;j<J;j++)
	{
		if (dist_fst[j]<=quantile_fst[j]  && nb_fst[j]<=nb_final)
			write_line_fst[j] << endl;
	}

	// write out data if file is not already in use by an another thread
	if (!pthread_mutex_trylock(&mutex_outfile))
	{
		for (int j=0;j<J;j++)
			outfile_fis[j] << write_line_fis[j].str();
		for (int j=0;j<J;j++)
			outfile_fst[j] << write_line_fst[j].str();
		pthread_mutex_unlock( &mutex_outfile );

		for (int j=0;j<J;j++)
			write_line_fis[j].str("");
		for (int j=0;j<J;j++)
			write_line_fst[j].str("");

		if (nb_waited>max_wait)
			max_wait=nb_waited;

		nb_waited=0;
	}
	// force to wait in the queue if waiting for too long
	else if (nb_waited>20)
	{
		pthread_mutex_lock( &mutex_outfile );
		for (int j=0;j<J;j++)
			outfile_fis[j] << write_line_fis[j].str();
		for (int j=0;j<J;j++)
			outfile_fst[j] << write_line_fst[j].str();
		pthread_mutex_unlock( &mutex_outfile );
		for (int j=0;j<J;j++)
			write_line_fis[j].str("");
		for (int j=0;j<J;j++)
			write_line_fst[j].str("");
		nb_waited=0;
	}
	else
	{
		nb_waited++;
	}

}
else if (pilot) // for pilot runs
{
	pthread_mutex_lock( &mutex_counter  );
	if (local_rep<nbreps)
	{
        for (int j=0;j<J;j++)
            v_dist_fis[j][local_rep]=dist_fis[j];
        for (int j=0;j<J;j++)
            v_dist_fst[j][local_rep]=dist_fst[j];
	}
	pthread_mutex_unlock( &mutex_counter  );
}

if (pilot)
{
pthread_mutex_lock( &mutex_gauge );
        // ecrit le compteur si necessaire
        if  ((100*rep/nbreps)>last_printf)
        {
                        last_printf=100*rep/nbreps;
                        cout << " " <<  last_printf << "%";
        }
pthread_mutex_unlock( &mutex_gauge );
}

if (!pilot)
{
pthread_mutex_lock( &mutex_gauge );
        int min=nb_final;
        for (int j=0;j<J;j++)
        {
            if (nb_fis[j]<min) min=nb_fis[j];
            if (nb_fis[j]<min) min=nb_fst[j];
        }
        // ecrit le compteur si necessaire
        if  ((100*min/nb_final)>last_printf)
        {
                        last_printf=100*min/nb_final;
                        cout << " " <<  last_printf << "%";
        }
pthread_mutex_unlock( &mutex_gauge );
}

}

// flush temp storage of data
if (!pilot)
{
    pthread_mutex_lock( &mutex_outfile );
    for (int j=0;j<J;j++)
        outfile_fis[j] << write_line_fis[j].str();
    for (int j=0;j<J;j++)
        outfile_fst[j] << write_line_fst[j].str();
    pthread_mutex_unlock( &mutex_outfile );
}

return NULL;

}




void read_input(ifstream& infile)
{

ofstream outfile;
outfile.open((odir+file_prefix+"_check-input.txt").c_str());
outfile << "File to check that the input file is read correctly." << endl << endl;

outfile << "There are " << nb_class_fis << " quantiles for each population used as summary statistics for Fis." << endl ;
outfile << "There are "<< nb_class_fst << " quantiles for each population used as summary statistics for each Fst." << endl ;
outfile << "There are "<< nb_final << " outputs." << endl ;
outfile << "Acceptance ration is " << ratio << endl << endl ;

outfile << "Output file names prefix is \"" << file_prefix << "\"" <<  endl;
outfile << "Output file directory is \"" << odir << "\"" <<  endl;
if( opt->getValue( "s" ) != NULL )
    outfile << "Output file names prefix for appending statistics is \"" << stats_prefix << "\"" << endl << endl;
outfile << "Input file is \"" << file_name << "\"" << endl << endl;

string line;

while(getline(infile,line,'=') )
{
if (line.length()>=6 && line.substr(line.length()-6,line.length())=="[loci]") //read nb of loci
{
getline(infile,line);
istringstream read_line(line);
read_line >> I;
outfile << "There are " << I << " loci." << endl << endl;
}

else if (line.length()>=13 && line.substr(line.length()-13,line.length())=="[populations]") //read nb of populations
{
getline(infile,line);
istringstream read_line(line);
read_line >> J;
outfile << "There are " << J << " populations." << endl << endl;
}

else if (line.length()>=5 && line.substr(line.length()-5,line.length())=="[pop]") //read allele count
{
getline(infile,line);
istringstream read_line(line);
int j;
read_line >> j; // population index
j=j-1;
for(int i=0;i<I;i++) // locus index
        {
        getline(infile,line); // read the line of allele count at locus i
        istringstream read_line(line);

        int locus_index;  // check the locus index is correct
        read_line >> locus_index;
        if(locus_index!=i+1) {fprintf(stderr, "Could not find allele count for population %d at locus %d\n",j+1,i+1);  exit(EXIT_FAILURE);}

        read_line >> pop_target[j].locus[i].n;

        read_line >> pop_target[j].locus[i].nA1;

        }
}

}

// write allele counts
outfile << "Allele counts:" << endl;
for(int j=0;j<J;j++)
{
        for(int i=0;i<I;i++)
        {
                outfile << "Pop. " << j+1 << " locus " << i+1 << " : " ;
                outfile << setw(3) << pop_target[j].locus[i].n << " " << pop_target[j].locus[i].nA1;
                outfile << endl;
        }
        outfile << endl;
}


infile.close();
outfile.close();

// calculate sum of all populations
int sum_pops[max_nr_loci];
int sum_pops_indiv[max_nr_loci];

for (int i=0;i<I;i++)
    {
    sum_pops[i]=0;
    sum_pops_indiv[i]=0;
    for (int j=0;j<J;j++)
        {
        sum_pops[i]+=pop_target[j].locus[i].nA1;
        sum_pops_indiv[i]+=pop_target[j].locus[i].n;
        }
    }

double freq[max_nr_loci]; // for quantile estimation
// calculate summary statistics for Fis
//if(!opt->getFlag( "comfis" ))
//{
	for (int j=0;j<J;j++)
	{
    		int i2=0;
    		for (int i=0;i<I;i++)
        		{
                    		freq[i2]=((double)pop_target[j].locus[i].nA1)/((double)pop_target[j].locus[i].n);
                    		i2++;
        		}

		    sort(&freq[0],&freq[I]);

    		for (int k=0;k<nb_class_fis;k++)
    		{
        		target_fis[k+j*nb_class_fis]=freq[(int)(I*(k+1)/(nb_class_fis+1))];

    		}

	}
/*}
else
{
	for (int j=0;j<J;j++)
	{
    		int i2=0;
    		for (int i=0;i<I;i++)
        		{
                    		freq[i2]=(((double)sum_pops[i])/((double)sum_pops_indiv[i]));
                    		i2++;
        		}

		    sort(&freq[0],&freq[I]);

    		for (int k=0;k<nb_class_fis;k++)
    		{
        		target_fis[k+j*nb_class_fis]=freq[(int)(I*(k+1)/(nb_class_fis+1))];
    		}
	}
}*/

// calculate sum stats for Fst
// calculate difference between pop_j and sum_pops
for (int j=0;j<J;j++)
{
    int i2=0;
    for (int i=0;i<I;i++)
        {
                if(sum_pops[i]!=0)
                    freq[i2]=(((double)pop_target[j].locus[i].nA1)/((double)pop_target[j].locus[i].n) - ((double)sum_pops[i])/((double)sum_pops_indiv[i])) / (((double)sum_pops[i])/((double)sum_pops_indiv[i])) ;
                else freq[i2]=0;
                i2++;
        }

    sort(&freq[0],&freq[I]);

    for (int k=0;k<nb_class_fst;k++)
    {
        target_fst[k+j*nb_class_fst]=freq[(int)(I*(k+1)/(nb_class_fst+1))];
	//cout << target_fst[k+j*nb_class_fst]<< " ";
	//target_fst[k+j*nb_class_fst]=target_fis[k+j*nb_class_fis];
    }
//cout << endl;
}


}


void calculate_stats(float *res_vect,double hpdi_pc,int nb_final,double *rm,double *rvar,double *rhpdi_low,double *rhpdi_high,double *rmode)
{
    double m=0; // mean
    double var=0.0; // variance
    double hpdi_low,hpdi_high;
    double mode;

    int orderindex,orderindex2;
    double *order;
    double min=1.7E+308; // min value
    double max=-1.7E+308; // max value

    // for fis:
    // mean
    for (int l=0;l<nb_final;l++)
    {
        m+=res_vect[l];
        if (res_vect[l]>max) max=res_vect[l];
        if (res_vect[l]<min) min=res_vect[l];
    }
    m=m/nb_final;

    // var
    for (int l=0;l<nb_final;l++)
            var+=(res_vect[l]-m)*(res_vect[l]-m);
    var=var/(nb_final-1);

    // sort data for HPDI
    sort(&res_vect[0],&res_vect[nb_final]);
    orderindex=(1 > (int)ceil((1.0-hpdi_pc)*nb_final-0.000000001)) ? 1 : (int)ceil((1.0-hpdi_pc)*nb_final-0.000000001);
    order=new double[orderindex];//(double*) malloc( orderindex*sizeof(double));
    for (int j=0;j<orderindex;j++)
        order[j]=res_vect[j+nb_final-orderindex]-res_vect[j];
    orderindex2=minindex(order,orderindex);

    hpdi_low=res_vect[orderindex2];
    hpdi_high=res_vect[nb_final-orderindex+orderindex2];

    // mode estimation via density
    double lambda=1.06*sqrt(var)*pow(nb_final,-0.2);
    if (lambda==0) lambda=0.01;

    double x[200];
    double y[200];
    for (int k=0;k<200;k++)
    {
        x[k]=min+k*(max-min)/200;
        y[k]=0;
    }
    for (int k=0;k<nb_final;k++)
    {
        for(int l=0;l<200;l++)
              y[l]=y[l]+exp(-((x[l]-res_vect[k])/lambda)*((x[l]-res_vect[k])/lambda)/2)/sqrt(2*Pi);
    }

    int mode_index=0;
    for (int k=1;k<200;k++)
    {
        if (y[k]>y[mode_index])
            mode_index=k;
    }

    mode=x[mode_index];

    *rm=m;
    *rvar=var;
    *rhpdi_low=hpdi_low;
    *rhpdi_high=hpdi_high;
    *rmode=mode;
}
